; KHS-NES-Brainfuck (Brainfuck-tulkki). Knnetn NESASM:illa.

; Makrot
	macro LdaSta
		lda \1
		sta \2
	endm
	macro WaitVBL
		bit PPU_STATUS
		Loop\@:
			bit PPU_STATUS
			bpl Loop\@
	endm

; Laitteistorekisterit
PPU_CTRL1  = $2000
PPU_CTRL2  = $2001
PPU_STATUS = $2002
PPU_ADDR   = $2006
PPU_DATA   = $2007
SPR_DMA    = $4014
JOYPAD1    = $4016

; Peliohjaimen nappien koodit
BTA  = %10000000
BTB  = %01000000
BTSE = %00100000
BTST = %00010000
BTU  = %00001000
BTD  = %00000100
BTL  = %00000010
BTR  = %00000001

; Muuttujat
OhjelTila  = $00   ; ohjelman tila (0 = editointi, 1 = ajo, 2 = sytteen kysyminen kyttjlt)
OhjainTila = $01   ; peliohjaimen tila
OhjelPit   = $02   ; Brainfuck-ohjelman pituus (0 - 255)
EpaSuoOsL  = $03   ; osoite epsuoria osoitustapoja kytettess (alempi tavu)
EpaSuoOsH  = $04   ; osoite epsuoria osoitustapoja kytettess (ylempi tavu)
Puskuri    = $05   ; tulostuspuskuri (1 tavu)
PuskPit    = $06   ; tulostuspuskurin pituus (0 - 1)
MerkTulost = $07   ; montako merkki Brainfuck-ohjelma on tulostanut
SyoteX     = $08   ; virtuaalinppimist - merkin X (0 - 15)
SyoteY     = $09   ; virtuaalinppimist - merkin Y (0 - 5)
SyoteMerk  = $0A   ; virtuaalinppimist - merkin koodi (32 - 127)
Temp       = $0B   ; tilapismuistipaikka

; Muistialueet, joita Brainfuck-ohjelma kytt
BFkoodi1 = $0200   ; koodi vlilyntien kanssa (255 tavua)
BFkoodi2 = $0300   ; koodi ilman vlilyntej (255 tavua)
BFsulut  = $0400   ; sulkujen eli [:jen ja ]:jen vastakskyjen osoitteet (255 tavua)
BFRAM    = $0500   ; RAM (256 tavua)

Spritet  = $0600   ; 256 tavua

; Header
	inesprg 1
	ineschr 0
	inesmir 0
	inesmap 0

; *****************************************************************************

; Ohjelma alkaa
	bank 0
	org $C000
OhjelmanAlku:

	lda #$00
	sta PPU_CTRL1
	sta PPU_CTRL2
	sta <OhjainTila
	sta <OhjelPit

	tax
	BFkoodiTyhj:
		sta BFkoodi1, x
		inx
		bne BFkoodiTyhj

	; Spritejen alkuarvot
	SprSilm:
		LdaSta SpritetAlku, x, Spritet, x
		inx
		cpx #8
		bne SprSilm

	; Muut spritet piiloon
	lda #$FF
	SprTayttoSilm:
		sta Spritet, x
		inx
		bne SprTayttoSilm

	WaitVBL
	OdotaToinenVBL:
		bit PPU_STATUS
		bpl OdotaToinenVBL

; Paletti
	; Tyt mustalla
	lda #$3F
	ldx #$00
	jsr AsetaPPUosoiteAX
	lda #$0F   ; musta
	ldx #32
	jsr TulostaMerkkia

	; Muuta kaksi vri valkoisiksi
	lda #$3F
	ldx #$01
	jsr AsetaPPUosoiteAX
	ldy #$30   ; valkoinen
	sty PPU_DATA
	ldx #$11
	jsr AsetaPPUosoiteAX
	sty PPU_DATA

; CHR-RAM
	LdaSta #high(CHRdata), <EpaSuoOsH
	ldy #$00
	sty <EpaSuoOsL
	sty PPU_ADDR
	sty PPU_ADDR
	CHRkopSilm:
		LdaSta [EpaSuoOsL], y, PPU_DATA
		iny
		; Joka merkin 2. bittitaso tyhj
		tya
		and #%00000111
		bne EiTyhjaa
			ldx #8
			TyhjaSilm:
				sta PPU_DATA
				dex
				bne TyhjaSilm
			EiTyhjaa:
		cpy #0
		bne CHRkopSilm
			; Vaihdetaan osoitteen enemmn merkitsev tavu
			inc <EpaSuoOsH
			lda <EpaSuoOsH
			cmp #high(CHRdata) + 8
			bne CHRkopSilm

	LdaSta #high(AjoTila), <EpaSuoOsH

PaluuEditointitilaan:
	lda #$00
	sta PPU_CTRL1
	sta PPU_CTRL2
	sta <OhjelTila
	sta <SyoteX
	sta <SyoteY

; Name Table
	lda #$20
	ldx #$00
	jsr AsetaPPUosoiteAX
	ldx #RLESIJ1
	jsr TulostaRLEdataa
	ldx #0
	NameSilm:
		LdaSta BFkoodi1, x, PPU_DATA
		inx
		bne NameSilm
	ldx #RLESIJ2
	jsr TulostaRLEdataa

; Attribute Table
	lda #%00000000
	ldx #64
	jsr TulostaMerkkia

; Viel kursori Name Tableen
	lda #$22
	ldx <OhjelPit
	jsr AsetaPPUosoiteAX
	LdaSta #'_', PPU_DATA

	jsr NollaaPPUosoite
	WaitVBL

OdotaAjonAlkamista1:
	LdaSta #%10000000, PPU_CTRL1
	LdaSta #%00001010, PPU_CTRL2

; Odotetaan, ett NMI:ss poistutaan editointitilasta
OdotaAjonAlkamista2:
	lda <OhjelTila
	beq OdotaAjonAlkamista2

; *****************************************************************************

; Brainfuck-ohjelman ajaminen alkaa
	LdaSta #%00000000, PPU_CTRL1

; Kopioidaan Brainfuck-koodin alkupuoli nyttmuistista BFkoodi1:een
	WaitVBL
	lda #$22
	ldx #$00
	jsr AsetaPPUosoiteAX
	lda PPU_DATA
	KoodiKopSilm1:
		LdaSta PPU_DATA, BFkoodi1, x
		inx
		bpl KoodiKopSilm1
	jsr NollaaPPUosoite

; Kopioidaan Brainfuck-koodin loppupuoli nyttmuistista BFkoodi1:een
	WaitVBL
	lda #$22
	ldx #$80
	jsr AsetaPPUosoiteAX
	lda PPU_DATA
	KoodiKopSilm2:
		LdaSta PPU_DATA, BFkoodi1, x
		inx
		bne KoodiKopSilm2
	jsr NollaaPPUosoite

; Tyhjennetn BFkoodi2
	lda #$00
	tax
	TyhjSilm:
		sta BFkoodi2, x
		inx
		bne TyhjSilm

; Kopioidaan BFkoodi1:n sislt BFkoodi2:een ilman vlilyntej
	tay
	KoodiKopSilm3:
		lda BFkoodi1, x
		cmp #' '
		beq EiOteta
			sta BFkoodi2, y
			iny
			EiOteta:
		inx
		bne KoodiKopSilm3

; Otetaan muistiin kustakin sulusta eli [:sta tai ]:sta, miss sit vastaava ] tai [ sijaitsee.
; Y: silmukkalaskuri. Pino: aukinaisten sulkujen osoitteet.
	ldy #0
	dex
	txs
	SulkuKeruuSilm:
		lda BFkoodi2, y

		cmp #'['
		bne EiAlkusulku
			; Nykyisen kskyn osoite pinoon
			tya
			pha
			jmp MerkkiTulkittu
			EiAlkusulku:

		cmp #']'
		bne MerkkiTulkittu
			; Nykyisen kskyn vastakskyksi edellinen avattu sulku, ja pinvastoin
			pla
			tsx
			beq PinonAlivuoto
			sta BFsulut, y
			tax
			tya
			sta BFsulut, x
			MerkkiTulkittu:

		iny
		bne SulkuKeruuSilm
		dey   ; kertoo, ett alivuotoa ei tapahtunut
		PinonAlivuoto:

; Onko alku- ja loppusulkuja sama mr
	tsx
	inx
	beq SulutKunnossa
		; Nytetn virheilmoitus ja odotetaan napinpainallusta
		WaitVBL
		lda #$23
		ldx #$20
		jsr AsetaPPUosoiteAX
		ldx #TEKSTISIJ1
		iny
		beq EiAlivuotoa
			ldx #TEKSTISIJ2
			EiAlivuotoa:
		jsr TulostaTekstidataa
		jsr NollaaPPUosoite
		OdotaB1:
			jsr LueOhjain
			sta <OhjainTila
			and #BTB
			beq OdotaB1

		; Palataan editointitilaan
		dec <OhjelTila
		WaitVBL
		lda #$23
		ldx #$20
		jsr AsetaPPUosoiteAX
		lda #' '
		ldx #32
		jsr TulostaMerkkia
		jsr NollaaPPUosoite
		jmp OdotaAjonAlkamista1

		SulutKunnossa:

	LdaSta #%00000000, PPU_CTRL2

; Vaihdetaan kunkin BFkoodi2:n kskyn tilalle osoite, jossa kskyn suorittava ohjelmanptk on.
; Kursori eli "_" on ksky, joka lopettaa ohjelman.
	ldx #0
	TulkintaSilm1:
		lda BFkoodi2, x
		ldy #0
		TulkintaSilm2:
			cmp Kaskyt, y
			beq KaskyTulkittu
			iny
			cpy #10
			bne TulkintaSilm2
			KaskyTulkittu:
		LdaSta KaskyOsoitteet, y, BFkoodi2, x
		inx
		bne TulkintaSilm1

; Tyhjennetn Brainfuck-RAM
	txa
	BFRAMtyhj:
		sta BFRAM, x
		inx
		bne BFRAMtyhj

; Kirjoitetaan Name Table uudelleen
	lda #$20
	jsr AsetaPPUosoiteAX
	ldx #RLESIJ3
	jsr TulostaRLEdataa
	ldx #TEKSTISIJ3
	jsr TulostaTekstidataa
	lda #' '
	ldx #28
	jsr TulostaMerkkia

	; Virtuaalinppimist
	ldx #32   ; ASCII-koodi
	VirtuSilm:
		txa
		and #%00001111
		bne EiRivinvaihtoa
			lda #' '
			ldy #16
			VirtuValiSilm:
				sta PPU_DATA
				dey
				bne VirtuValiSilm
			EiRivinvaihtoa:
		stx PPU_DATA
		inx
		bpl VirtuSilm

	lda #' '
	ldx #136
	jsr TulostaMerkkia

; Attribute Tablessa virtuaalinppimist piiloon
	lda #%01010101
	jsr AsetaVirtuNappTila

	jsr NollaaPPUosoite
	WaitVBL

	LdaSta #%10000000, PPU_CTRL1
	LdaSta #%00001010, PPU_CTRL2

; *****************************************************************************

; Varsinainen Brainfuck-koodin suoritus
	jmp AjoTila
	org $C300
AjoTila:

	ldx #$00
	stx <PuskPit
	stx <MerkTulost

; Y: osoite Brainfuck-koodissa. X: osoite Brainfuck-RAM:issa.
	ldy #$FF
SuoritusSilm:
	iny
	LdaSta BFkoodi2, y, <EpaSuoOsL
	jmp [EpaSuoOsL]

	KaksiPlus:
		inc BFRAM, x
	Plus:
		inc BFRAM, x
		jmp SuoritusSilm

	KaksiMiinus:
		dec BFRAM, x
	Miinus:
		dec BFRAM, x
		jmp SuoritusSilm

	Vasen:
		dex
		jmp SuoritusSilm

	Oikea:
		inx
		jmp SuoritusSilm

	Alkusulku:
		lda BFRAM, x
		bne SuoritusSilm
			lda BFsulut, y
			tay
		jmp SuoritusSilm

	Loppusulku:
		lda BFRAM, x
		beq SuoritusSilm
			lda BFsulut, y
			tay
		jmp SuoritusSilm

	Piste:
		; Laitetaan merkki puskuriin ja odotetaan, ett se tyhjennetn NMI:ss
		LdaSta BFRAM, x, <Puskuri
		inc <PuskPit
		OdotaPuskTyhj:
			lda <PuskPit
			bne OdotaPuskTyhj
		lda <MerkTulost
		beq PoistuOhjelmasta   ; jos tulostettu 256 merkki
		jmp SuoritusSilm

	Pilkku:
		stx <Temp

		; Teksti "Character?"
		WaitVBL
		lda #$22
		ldx #$40
		jsr AsetaPPUosoiteAX
		ldx #TEKSTISIJ4
		jsr TulostaTekstidataa

		; Virtuaalinppimist nkyviin
		lda #%00000000
		jsr AsetaVirtuNappTila
		jsr NollaaPPUosoite

		; Spritet nkyviin
		LdaSta #%00011110, PPU_CTRL2

		; Kyttjlt kysytn syte NMI:ss
		inc <OhjelTila
		OdotaNMIta:
			ldx <OhjelTila
			dex
			bne OdotaNMIta

		; Teksti "Running..."
		lda #$22
		ldx #$40
		jsr AsetaPPUosoiteAX
		ldx #TEKSTISIJ3
		jsr TulostaTekstidataa

		; Virtuaalinppimist piiloon
		lda #%01010101
		jsr AsetaVirtuNappTila
		jsr NollaaPPUosoite

		; Spritet piiloon
		LdaSta #%00001010, PPU_CTRL2

		ldx <Temp
		LdaSta <SyoteMerk, BFRAM, x
		jmp SuoritusSilm

	PoistuOhjelmasta:

; Brainfuck-ohjelma on ajettu
	LdaSta #%00000000, PPU_CTRL1
	WaitVBL

	; Teksti "Finished."
	lda #$22
	ldx #$40
	jsr AsetaPPUosoiteAX
	ldx #TEKSTISIJ5
	jsr TulostaTekstidataa
	jsr NollaaPPUosoite

	; Odota, ett painetaan B
	OdotaB2:
		jsr LueOhjain
		sta <OhjainTila
		and #BTB
		beq OdotaB2

	jmp PaluuEditointitilaan

; *****************************************************************************

NMI:
	php
	pha
	txa
	pha
	tya
	pha

	ldx <OhjelTila
	beq NMI_editointitila
	dex
	beq NMI_ajotila
	jmp NMI_syotetila

	NMI_editointitila:
		jsr LueOhjain
	
		; Ei jatketa, jos nappien tila on sama kuin edellisell kerralla
		cpx <OhjainTila
		bne Jatka1
		jmp PoistuNMIsta
		Jatka1:
		stx <OhjainTila
	
		; Jos kirjoitettuja merkkej on alle 255 ja on painettu nappia, jolla listn merkki,
		; listn kyseinen merkki ja poistutaan NMI:st.
		ldx <OhjelPit
		inx
		beq NapitLuettu1
			ldy #10
			TutkiNappiSilm:
				lda KaskyNapit, y
				cmp <OhjainTila
				bne EiOllutTamaNappi
					lda #$22
					ldx <OhjelPit
					jsr AsetaPPUosoiteAX
					LdaSta Kaskyt, y, PPU_DATA
					LdaSta #'_', PPU_DATA
					inc <OhjelPit
					jmp NollaaJaPoistuNMIsta
					EiOllutTamaNappi:
				dey
				bpl TutkiNappiSilm
			NapitLuettu1:
	
		; Jos on painettu start-vasen ja kirjoitettu ainakin yksi merkki, poistetaan viimeinen
		; ja poistutaan NMI:st.
		lda <OhjelPit
		beq EiBackSpacea
		lda <OhjainTila
		cmp #BTST|BTL
		bne EiBackSpacea
			dec <OhjelPit
			lda #$22
			ldx <OhjelPit
			jsr AsetaPPUosoiteAX
			LdaSta #'_', PPU_DATA
			LdaSta #' ', PPU_DATA
			jmp NollaaJaPoistuNMIsta
			EiBackSpacea:
	
		; Jos on painettu select-start, ajetaan ohjelma
		lda <OhjainTila
		cmp #BTSE|BTST
		bne EiAjeta
			inc <OhjelTila
			EiAjeta:
	
		jmp PoistuNMIsta

	NMI_ajotila:
		jsr LueOhjain
		sta <OhjainTila

		; Jos on painettu B, keskeytetn
		and #BTB
		beq EiKeskeyteta
			jmp PaluuEditointitilaan
			EiKeskeyteta:

		; Jos puskurissa on merkki, tulostetaan se
		lda <PuskPit
		bne Jatka2
		jmp PoistuNMIsta
		Jatka2:
			lda <Puskuri
			cmp #$0A
			beq Rivinvaihto
				lda #$21
				ldx <MerkTulost
				jsr AsetaPPUosoiteAX
				LdaSta <Puskuri, PPU_DATA
				inc <MerkTulost
				dec <PuskPit
				jmp NollaaJaPoistuNMIsta
			Rivinvaihto:
				lda <MerkTulost
				and #%11100000
				adc #31
				sta <MerkTulost
				dec <PuskPit
				jmp PoistuNMIsta
	
	NMI_syotetila:
		jsr LueOhjain
	
		; Ei reagoida nappeihin, jos niiden tila on sama kuin edellisell kerralla
		cmp <OhjainTila
		beq NapitLuettu
		sta <OhjainTila

		lsr a
		bcs Oikea2
		lsr a
		bcs Vasen2
		lsr a
		bcs Ala
		lsr a
		bcs Yla
		lsr a
		lsr a
		lsr a
		bcs Pois
		lsr a
		bcs Valitse

		jmp NapitLuettu

		Vasen2:
			ldx <SyoteX
			dex
			txa
			and #%00001111
			sta <SyoteX
			jmp NapitLuettu

		Oikea2:
			ldx <SyoteX
			inx
			txa
			and #%00001111
			sta <SyoteX
			jmp NapitLuettu

		Yla:
			ldx <SyoteY
			dex
			bpl EiTaysille
				ldx #5
				EiTaysille:
			stx <SyoteY
			jmp NapitLuettu

		Ala:
			ldx <SyoteY
			inx
			cpx #6
			bne EiNollaan
				ldx #0
				EiNollaan:
			stx <SyoteY
			jmp NapitLuettu

		Valitse:
			dec <OhjelTila
			jmp NapitLuettu

		Pois:
			jmp PaluuEditointitilaan

		NapitLuettu:

		; Spritejen Y
		lda <SyoteY
		asl a
		asl a
		asl a
		tax
		adc #$9F
		sta Spritet + 0
		sta Spritet + 4

		; Spriten 0 kuva ja SyoteMerk
		txa
		asl a
		adc #$20
		adc <SyoteX
		sta Spritet + 1
		cmp #127
		bne EiVirtuNappEnter
			lda #$0A
			EiVirtuNappEnter:
		sta <SyoteMerk

		; Spritejen X
		lda <SyoteX
		asl a
		asl a
		asl a
		adc #$40
		sta Spritet + 3
		sta Spritet + 7

		LdaSta #high(Spritet), SPR_DMA

	NollaaJaPoistuNMIsta:
		jsr NollaaPPUosoite
	PoistuNMIsta:
		pla
		tay
		pla
		tax
		pla
		plp
		rti

; *****************************************************************************

NollaaPPUosoite:
	LdaSta #$00, PPU_ADDR
	sta PPU_ADDR
	rts

AsetaPPUosoiteAX:
	sta PPU_ADDR
	stx PPU_ADDR
	rts

TulostaMerkkia:
	; Tulostaa A:n X kertaa
	MerkTulSilm:
		sta PPU_DATA
		dex
		bne MerkTulSilm
	rts

TulostaTekstidataa:
	; Tulostaa taulukosta TekstiData tavut X:st seuraavaan loppumerkkiin ($00).
	TekstiSilm:
		lda TekstiData, x
		beq LoppumerkkiHavaittu1
		sta PPU_DATA
		inx
		bne TekstiSilm
	LoppumerkkiHavaittu1:
	rts

TulostaRLEdataa:
	; Tulostaa taulukosta RLEdata tavut X:st seuraavaan loppumerkkiin.
	; RLE-data on jaettu lohkoihin. Lohkon 1. tavu kertoo lohkon tyypin:
	;     $00       = loppumerkki
	;     $01 - $7F = pakattu,     pituus 2 - 128 tavua
	;     $80 - $FF = pakkaamaton, pituus 1 - 128 tavua
	; Lohkon muut tavut ovat varsinainen data (pakatussa lohkossa aina yksi tavu).
	RLEsilm1:
		ldy RLEdata, x   ; lohkon tyyppi
		beq LoppumerkkiHavaittu2
		bpl Pakattu
			; Pakkaamaton lohko
			RLEsilm2:
				inx
				LdaSta RLEdata, x, PPU_DATA
				dey
				bmi RLEsilm2
			jmp LohkoKopioitu
		Pakattu:
			inx
			lda RLEdata, x
			RLEsilm3:
				sta PPU_DATA
				dey
				bpl RLEsilm3	
		LohkoKopioitu:
		inx
		bne RLEsilm1
	LoppumerkkiHavaittu2:
	rts

LueOhjain:
	; Luetaan peliohjaimen tila A:han ja X:n.
	; Bitit: A, B, select, start, yl, ala, vasen, oikea.
	ldx #1
	stx JOYPAD1
	dex
	stx JOYPAD1
	ldy #8
	OhjainLukuSilm:
		lda JOYPAD1
		ror a
		txa
		rol a
		tax
		dey
		bne OhjainLukuSilm
	rts

AsetaVirtuNappTila:
	; Kirjoittaa Attribute Tableen virtuaalinppimistn kohdalle A:n arvoa
	ldx #$23
	stx PPU_ADDR
	ldx #$EA
	stx PPU_ADDR
	ldx #12
	VirtuNappSilm:
		sta PPU_DATA
		dex
		bne VirtuNappSilm
	rts

; *****************************************************************************

; Brainfuck-kskyt; mill napilla ne kirjoitetaan; kskyt suorittavien koodinptkien osoitteet
Kaskyt:     db '+', '-', '<', '>', '[', ']', '.',      ',',      $8A,      $8B,      ' '
KaskyNapit: db BTU, BTD, BTL, BTR, BTB, BTA, BTSE|BTA, BTSE|BTB, BTST|BTU, BTST|BTD, BTST|BTR
KaskyOsoitteet:
HY func \1 - AjoTila
	db HY(Plus), HY(Miinus), HY(Vasen), HY(Oikea), HY(Alkusulku), HY(Loppusulku)
	db HY(Piste), HY(Pilkku), HY(KaksiPlus), HY(KaksiMiinus)
	db HY(PoistuOhjelmasta)   ; ei vastaa vlilynti vaan "_":aa

; Editointitilan ja ajotilan Name Table RLE-pakattuina.
; Lohkojen tyyppitavut:
LOE func $80 + \1 - 1   ; pakkaamaton lohko (pituus 1 - 128)
LOP func \1 - 1         ; pakattu lohko (pituus 2 - 128)
LOLO = $00              ; loppumerkki
RLEdata:
	; Editointitila ennen Brainfuck-koodia
	RLEdata1:
	db LOP(102),' ', LOE(1) ,$82
	db LOP(17), $80, LOE(1) ,$83
	db LOP(13), ' ', LOE(19),$81,"KHS-NES-Brainfuck",$81
	db LOP(13), ' ', LOE(1) ,$84
	db LOP(17), $80, LOE(1) ,$85
	db LOP(40), ' ', LOE(8) ,$86,"=+  ",$87,"=-"
	db LOP(3) , ' ', LOE(8) ,$88,"=<  ",$89,"=>"
	db LOP(3) , ' ', LOE(8) ,"B=[  A=]"
	db LOP(35), ' ', LOE(9) ,"start-",$86,"=",$8A
	db LOP(3),  ' ', LOE(13),"start-",$89,"=space"
	db LOP(7),  ' ', LOE(9) ,"start-",$87,"=",$8B
	db LOP(3),  ' ', LOE(41),"start-",$88,"=backspace  select-B=,  select-A=."
	db LOP(42), ' ', LOE(16),"select-start=run"
	db LOP(47), ' ', LOP(32),$80
	db LOLO

	; Editointitila Brainfuck-koodin jlkeen
	RLEdata2:
	db LOP(32),$80, LOP(128),' ', LOP(32),' '
	db LOLO

	; Ajotila ennen "Running"-teksti
	RLEdata3:
	db LOP(128),' ', LOP(32) ,' ', LOE(7) ,"Output:"
	db LOP(57) ,' ', LOP(32) ,$80
	db LOP(128),' ', LOP(128),' ', LOP(32),$80
	db LOP(32) ,' '
	db LOLO

RLESIJ1 = RLEdata1 - RLEdata
RLESIJ2 = RLEdata2 - RLEdata
RLESIJ3 = RLEdata3 - RLEdata

; Yksittiset tekstit
TELO = $00   ; loppumerkki
TekstiData:
	TekstiData1: db "Error: '[' without ']'. Press B.", TELO
	TekstiData2: db "Error: ']' without '['. Press B.", TELO
	TekstiData3: db "Running... (B=end)          ", TELO
	TekstiData4: db "Character? (",$86,$87,$88,$89," A=OK B=end)", TELO
	TekstiData5: db "Finished. Press B.", TELO

TEKSTISIJ1 = TekstiData1 - TekstiData
TEKSTISIJ2 = TekstiData2 - TekstiData
TEKSTISIJ3 = TekstiData3 - TekstiData
TEKSTISIJ4 = TekstiData4 - TekstiData
TEKSTISIJ5 = TekstiData5 - TekstiData

SpritetAlku:
	db $FF, $00, %00000001, $FF   ; valittu merkki mustana
	db $FF, $8C, %00000000, $FF   ; pelkk valkoista

; *****************************************************************************

; CHR-RAM (256 grafiikkamerkki, 1 bittitaso, 2 kt)
	org $C800
CHRdata:
	incbin "KHS-NES-Brainfuck (1 bitplane).chr"

; *****************************************************************************

; NMI-, reset- ja IRQ-osoitteet
	bank 1
	org $FFFA
	dw NMI, OhjelmanAlku, 0